"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parse = void 0;
const htmlparser2_1 = require("htmlparser2");
const Factory_1 = require("./parser/Factory");
const FcAttributeHandler_1 = require("./parser/FcAttributeHandler");
const FcCallTagHandler_1 = require("./parser/FcCallTagHandler");
const FcContentAttributeHandler_1 = require("./parser/FcContentAttributeHandler");
const FcContentTagHandler_1 = require("./parser/FcContentTagHandler");
const FcEachTagHandler_1 = require("./parser/FcEachTagHandler");
const FcElseIfTagHandler_1 = require("./parser/FcElseIfTagHandler");
const FcElseTagHandler_1 = require("./parser/FcElseTagHandler");
const FcIfTagHandler_1 = require("./parser/FcIfTagHandler");
const FcKeyAttributeHandler_1 = require("./parser/FcKeyAttributeHandler");
const FcSkipChildrenHandler_1 = require("./parser/FcSkipChildrenHandler");
const FcTagHandler_1 = require("./parser/FcTagHandler");
const utils_1 = require("./parser/utils");
const DEFAULT_TAG_HANDLERS = {
    '*': new FcTagHandler_1.FcTagHandler(),
    'fc-each': new FcEachTagHandler_1.FcEachTagHandler(),
    'fc-if': new FcIfTagHandler_1.FcIfTagHandler(),
    'fc-else': new FcElseTagHandler_1.FcElseTagHandler(),
    'fc-else-if': new FcElseIfTagHandler_1.FcElseIfTagHandler(),
    'fc-content': new FcContentTagHandler_1.FcContentTagHandler(),
    'fc-call': new FcCallTagHandler_1.FcCallTagHandler()
};
const DEFAULT_ATT_HANDLERS = {
    '*': new FcAttributeHandler_1.FcAttHandler(),
    'fc-key': new FcKeyAttributeHandler_1.FcKeyAttHandler(),
    'fc-content': new FcContentAttributeHandler_1.FcContentAttHandler(),
    'fc-skip': new FcSkipChildrenHandler_1.FcSkipChildrenHandler(),
};
const DEFAULT_OPTIONS = {
    output: 'function',
    pretty: false,
    interpolation: /\{\{([\s\S]+?)\}\}/gm,
    propNamePrefix: '#',
    elVarName: 'el',
    ctxVarName: 'ctx',
    selfClosingElements: [
        'area', 'base', 'br', 'col', 'embed', 'hr', 'img', 'input',
        'keygen', 'link', 'meta', 'param', 'source', 'track', 'wbr'
    ],
    tagHandlers: DEFAULT_TAG_HANDLERS,
    attHandlers: DEFAULT_ATT_HANDLERS
};
/**
 * Parse an HTML template and return the factory of the underlying render function.
 * @param {string} html the HTML content to parse
 * @param {ParserOptions} options the options
 * @return {string | RenderFactory} the factory function as Function or string.
 */
function parse(html, options = {}) {
    options = utils_1.assign({}, DEFAULT_OPTIONS, options);
    options.tagHandlers = utils_1.assign({}, DEFAULT_TAG_HANDLERS, options.tagHandlers);
    options.attHandlers = utils_1.assign({}, DEFAULT_ATT_HANDLERS, options.attHandlers);
    const factory = new Factory_1.Factory(options);
    const p = new htmlparser2_1.Parser({
        onopentag(name, attrs) {
            const tagHandler = options.tagHandlers[name] ? options.tagHandlers[name] : options.tagHandlers['*'];
            if (tagHandler) {
                tagHandler.startTag(factory, name, attrs, options);
            }
        },
        onclosetag(name) {
            const tagHandler = options.tagHandlers[name] ? options.tagHandlers[name] : options.tagHandlers['*'];
            if (tagHandler) {
                tagHandler.endTag(factory, name, options);
            }
        },
        ontext(text) {
            factory.appendText(utils_1.interpolate(text, options));
        },
        oncomment(text) {
            factory.appendComment(utils_1.interpolate(text, options));
        }
    }, {
        xmlMode: false,
        decodeEntities: false,
        lowerCaseTags: true,
        lowerCaseAttributeNames: true,
        recognizeSelfClosing: true,
        recognizeCDATA: true
    });
    p.parseComplete(html);
    return factory.toFunction();
}
exports.parse = parse;
