import { Map, RenderFactory, RenderFunction } from './model';
import { RootElement } from './runtime/model';
/**
 * Update the sub tree of the given root according to a given factory of the render function.
 * @param {RenderFactory} factory the render factory
 * @param {HTMLElement} root the root element
 * @param {Map<any>} context the context
 * @return {RenderFunction} the render function
 */
export declare function createThenUpdate(factory: RenderFactory, root: HTMLElement, context?: Map<any>): RenderFunction;
/**
 * Update the sub tree of the given root according to the given render function.
 * @param {RenderFunction} render the render function
 * @param {RootElement} root the root element
 * @param {Context} context the context
 */
export declare function updateElement(render: RenderFunction, root: RootElement, context?: Map<any>): void;
/**
 * Open an element.
 *
 * @example <caption>set attributes</caption>
 * openElement('p', ['id', 'my-p' ,'class', classValue]);
 * closeElement();
 *
 * @example <caption>set properties</caption>
 * const selectValue = 'foo';
 * openElement('select', null, ['value', selectValue])
 * closeElement();
 *
 * @example <caption>set the element has a light DOM content</caption>
 * openElement('div', null, null, ['content', true])
 * closeElement();
 *
 * @example <caption>set the element's key</caption>
 * openElement('li', null, null, ['key', 'my-li'])
 * closeElement();
 *
 * @param {string} name the name of the element
 * @param {any[]} attrs the attributes of the element
 * @param {any[]} props the properties of the element
 * @param {any[]} opts the options driving the creation of the element, c.f. {@link ElementOptions}
 * @return {HTMLElement} the element
 */
export declare function openElement(name: string, attrs?: any[], props?: any[], opts?: any[]): HTMLElement;
/**
 * Close the current element.
 */
export declare function closeElement(): void;
/**
 * Open and close a void element.
 * @param {string} name the name of the element
 * @param {string[]} attrs the attributes of the element
 * @param {any[]} props the properties of the element
 * @param {any[]} opts the options driving the creation of the element, c.f. {@link ElementOptions}
 * @return {HTMLElement} the element
 */
export declare function voidElement(name: string, attrs?: string[], props?: any[], opts?: any[]): HTMLElement;
/**
 * To add a <code><fc-content></fc-content></code> node.
 * This node will be used to locate the entry of the light DOM structure.
 */
export declare function content(): void;
/**
 * To write a text node.
 * @param {string} value the text
 * @return {Node} the text node
 */
export declare function text(value?: string): Text;
/**
 * To write a comment node.
 * @param {string} value the comment
 * @return {Node} the comment node
 */
export declare function comment(value?: string): Comment;
